@@ -153,7 +153,7 @@ function testDataArgumentOnGeneratedCommand(data) {
 }
 
 function testDataEscapeOnGeneratedCommand(data) {
-  const paramsWin = `--data-raw "{""param1"":""value1"",""param2"":""value2""}"`;
+  const paramsWin = `--data-raw ^"{\\"param1\\":\\"value1\\",\\"param2\\":\\"value2\\"}^"`;
   const paramsPosix = `--data-raw '{"param1":"value1","param2":"value2"}'`;
 
   let curlCommand = Curl.generateCommand(data, "WINNT");
@@ -206,22 +206,22 @@ function testRemoveBinaryDataFromMultipartText(data) {
   ].join("");
 
   const EXPECTED_WIN_RESULT = [
-    '"',
+    '^"',
     boundary,
-    '"^\u000d\u000A\u000d\u000A"',
-    'Content-Disposition: form-data; name=""param1""',
-    '"^\u000d\u000A\u000d\u000A""^\u000d\u000A\u000d\u000A"',
+    "^\u000A\u000A",
+    'Content-Disposition: form-data; name=\\"param1\\"',
+    "^\u000A\u000A^\u000A\u000A",
     "value1",
-    '"^\u000d\u000A\u000d\u000A"',
+    "^\u000A\u000A",
     boundary,
-    '"^\u000d\u000A\u000d\u000A"',
-    'Content-Disposition: form-data; name=""file""; filename=""filename.png""',
-    '"^\u000d\u000A\u000d\u000A"',
+    "^\u000A\u000A",
+    'Content-Disposition: form-data; name=\\"file\\"; filename=\\"filename.png\\"',
+    "^\u000A\u000A",
     "Content-Type: image/png",
-    '"^\u000d\u000A\u000d\u000A""^\u000d\u000A\u000d\u000A"',
+    "^\u000A\u000A^\u000A\u000A",
     boundary + "--",
-    '"^\u000d\u000A\u000d\u000A"',
-    '"',
+    "^\u000A\u000A",
+    '^"',
   ].join("");
 
   if (Services.appinfo.OS != "WINNT") {
@@ -269,7 +269,7 @@ function testEscapeStringPosix() {
   const escapeChar = "'!ls:q:gs|ls|;ping 8.8.8.8;|";
   is(
     CurlUtils.escapeStringPosix(escapeChar),
-    "$'\\'\\041ls:q:gs|ls|;ping 8.8.8.8;|'",
+    "$'\\'\\041ls:q:gs^|ls^|;ping 8.8.8.8;^|'",
     "'!' should be escaped."
   );
 
@@ -295,62 +295,77 @@ function testEscapeStringPosix() {
     "$'\\xc3\\xa6 \\xc3\\xb8 \\xc3\\xbc \\xc3\\x9f \\xc3\\xb6 \\xc3\\xa9'",
     "Character codes outside of the decimal range 32 - 126 should be escaped."
   );
+
+  // Assert that ampersands are correctly escaped in case its tried to run on Windows
+  const evilCommand = `query=evil\n\ncmd & calc.exe\n\n`;
+  is(
+    CurlUtils.escapeStringPosix(evilCommand),
+    "$'query=evil\\n\\ncmd ^& calc.exe\\n\\n'",
+    "The evil command is escaped properly"
+  );
+
+  const str = "EvilHeader: &calc.exe&";
+  is(
+    CurlUtils.escapeStringPosix(str),
+    "'EvilHeader: ^&calc.exe^&'",
+    "The evil command is escaped properly"
+  );
 }
 
 function testEscapeStringWin() {
   const surroundedWithDoubleQuotes = "A simple string";
   is(
     CurlUtils.escapeStringWin(surroundedWithDoubleQuotes),
-    '"A simple string"',
+    '^"A simple string^"',
     "The string should be surrounded with double quotes."
   );
 
   const doubleQuotes = 'Quote: "Time is an illusion. Lunchtime doubly so."';
   is(
     CurlUtils.escapeStringWin(doubleQuotes),
-    '"Quote: ""Time is an illusion. Lunchtime doubly so."""',
+    '^"Quote: \\"Time is an illusion. Lunchtime doubly so.\\"^"',
     "Double quotes should be escaped."
   );
 
   const percentSigns = "%TEMP% %@foo% %2XX% %_XX% %?XX%";
   is(
     CurlUtils.escapeStringWin(percentSigns),
-    '"^%^TEMP^% ^%^@foo^% ^%^2XX^% ^%^_XX^% ^%?XX^%"',
+    '^"^%^TEMP^% ^%^@foo^% ^%^2XX^% ^%^_XX^% ^%?XX^%^"',
     "Percent signs should be escaped."
   );
 
   const backslashes = "\\A simple string\\";
   is(
     CurlUtils.escapeStringWin(backslashes),
-    '"\\\\A simple string\\\\"',
+    '^"\\\\A simple string\\\\^"',
     "Backslashes should be escaped."
   );
 
   const newLines = "line1\r\nline2\r\rline3\n\nline4";
   is(
     CurlUtils.escapeStringWin(newLines),
-    '"line1"^\r\n\r\n"line2"^\r\n\r\n""^\r\n\r\n"line3"^\r\n\r\n""^\r\n\r\n"line4"',
+    '^\"line1^\n\nline2^\n\n^\n\nline3^\n\n^\n\nline4^\"',
     "Newlines should be escaped."
   );
 
   const dollarSignCommand = "$(calc.exe)";
   is(
     CurlUtils.escapeStringWin(dollarSignCommand),
-    '"\\$(calc.exe)"',
+    '^"\\$(calc.exe)^"',
     "Dollar sign should be escaped."
   );
 
   const tickSignCommand = "`$(calc.exe)";
   is(
     CurlUtils.escapeStringWin(tickSignCommand),
-    '"\\`\\$(calc.exe)"',
+    '^"\\`\\$(calc.exe)^"',
     "Both the tick and dollar signs should be escaped."
   );
 
   const evilCommand = `query=evil\r\rcmd" /c timeout /t 3 & calc.exe\r\r`;
   is(
     CurlUtils.escapeStringWin(evilCommand),
-    '"query=evil"^\r\n\r\n""^\r\n\r\n"cmd"" /c timeout /t 3 & calc.exe"^\r\n\r\n""^\r\n\r\n""',
+    '^\"query=evil^\n\n^\n\ncmd\\\" /c timeout /t 3 & calc.exe^\n\n^\n\n^\"',
     "The evil command is escaped properly"
   );
 }
