@@ -10,6 +10,7 @@
 #include "mozilla/dom/WindowGlobalParent.h"
 #include "mozilla/dom/SyncedContextInlines.h"
 #include "mozilla/dom/BrowsingContext.h"
+#include "mozilla/dom/CloseWatcherManager.h"
 #include "mozilla/dom/Document.h"
 #include "mozilla/dom/UserActivationIPCUtils.h"
 #include "mozilla/PermissionDelegateIPCUtils.h"
@@ -18,7 +19,9 @@
 #include "mozilla/ClearOnShutdown.h"
 #include "nsGlobalWindowInner.h"
 #include "nsIScriptError.h"
+#include "nsIWebProgressListener.h"
 #include "nsIXULRuntime.h"
+#include "nsRFPTargetSetIDL.h"
 #include "nsRefPtrHashtable.h"
 #include "nsContentUtils.h"
 
@@ -75,6 +78,19 @@ bool WindowContext::IsInBFCache() {
   return TopWindowContext()->GetWindowStateSaved();
 }
 
+already_AddRefed<nsIRFPTargetSetIDL>
+WindowContext::GetOverriddenFingerprintingSettingsWebIDL() const {
+  Maybe<RFPTargetSet> overriddenFingerprintingSettings =
+      GetOverriddenFingerprintingSettings();
+  if (overriddenFingerprintingSettings.isNothing()) {
+    return nullptr;
+  }
+
+  nsCOMPtr<nsIRFPTargetSetIDL> protections =
+      new nsRFPTargetSetIDL(overriddenFingerprintingSettings.ref());
+  return protections.forget();
+}
+
 nsGlobalWindowInner* WindowContext::GetInnerWindow() const {
   return mWindowGlobalChild ? mWindowGlobalChild->GetWindowGlobal() : nullptr;
 }
@@ -121,7 +137,12 @@ void WindowContext::AppendChildBrowsingContext(
                         "Mismatched groups?");
   MOZ_DIAGNOSTIC_ASSERT(!mChildren.Contains(aBrowsingContext));
 
+  ClearLightDOMChildren();
+
   mChildren.AppendElement(aBrowsingContext);
+  if (!aBrowsingContext->IsEmbedderTypeObjectOrEmbed()) {
+    mNonSyntheticChildren.AppendElement(aBrowsingContext);
+  }
 
   // If we're the current WindowContext in our BrowsingContext, make sure to
   // clear any cached `children` value.
@@ -134,8 +155,10 @@ void WindowContext::RemoveChildBrowsingContext(
     BrowsingContext* aBrowsingContext) {
   MOZ_DIAGNOSTIC_ASSERT(Group() == aBrowsingContext->Group(),
                         "Mismatched groups?");
+  ClearLightDOMChildren();
 
   mChildren.RemoveElement(aBrowsingContext);
+  mNonSyntheticChildren.RemoveElement(aBrowsingContext);
 
   // If we're the current WindowContext in our BrowsingContext, make sure to
   // clear any cached `children` value.
@@ -144,6 +167,49 @@ void WindowContext::RemoveChildBrowsingContext(
   }
 }
 
+void WindowContext::UpdateChildSynthetic(BrowsingContext* aBrowsingContext,
+                                         bool aIsSynthetic) {
+  if (aIsSynthetic) {
+    mNonSyntheticChildren.RemoveElement(aBrowsingContext);
+  } else {
+    // The same BrowsingContext will be reused for error pages, so it can be in
+    // the list already.
+    if (!mNonSyntheticChildren.Contains(aBrowsingContext)) {
+      mNonSyntheticChildren.AppendElement(aBrowsingContext);
+    }
+  }
+}
+
+void WindowContext::ClearLightDOMChildren() {
+  mNonSyntheticLightDOMChildren.reset();
+}
+
+void WindowContext::EnsureLightDOMChildren() {
+  if (mNonSyntheticLightDOMChildren.isSome()) {
+    return;
+  }
+  mNonSyntheticLightDOMChildren.emplace();
+
+  for (const RefPtr<BrowsingContext>& bc : mNonSyntheticChildren) {
+    if (Element* el = bc->GetEmbedderElement(); el && el->IsInShadowTree()) {
+      continue;
+    }
+    mNonSyntheticLightDOMChildren->AppendElement(bc);
+  }
+}
+
+BrowsingContext* WindowContext::NonSyntheticLightDOMChildAt(uint32_t aIndex) {
+  EnsureLightDOMChildren();
+  return aIndex < mNonSyntheticLightDOMChildren->Length()
+             ? mNonSyntheticLightDOMChildren->ElementAt(aIndex).get()
+             : nullptr;
+}
+
+uint32_t WindowContext::NonSyntheticLightDOMChildrenCount() {
+  EnsureLightDOMChildren();
+  return mNonSyntheticLightDOMChildren->Length();
+}
+
 void WindowContext::SendCommitTransaction(ContentParent* aParent,
                                           const BaseTransaction& aTxn,
                                           uint64_t aEpoch) {
@@ -173,12 +239,6 @@ bool WindowContext::CanSet(FieldIndex<IDX_IsSecure>, const bool& aIsSecure,
   return CheckOnlyOwningProcessCanSet(aSource);
 }
 
-bool WindowContext::CanSet(FieldIndex<IDX_AllowMixedContent>,
-                           const bool& aAllowMixedContent,
-                           ContentParent* aSource) {
-  return CheckOnlyOwningProcessCanSet(aSource);
-}
-
 bool WindowContext::CanSet(FieldIndex<IDX_HasBeforeUnload>,
                            const bool& aHasBeforeUnload,
                            ContentParent* aSource) {
@@ -208,6 +268,24 @@ bool WindowContext::CanSet(FieldIndex<IDX_IsThirdPartyTrackingResourceWindow>,
   return CheckOnlyOwningProcessCanSet(aSource);
 }
 
+bool WindowContext::CanSet(FieldIndex<IDX_UsingStorageAccess>,
+                           const bool& aUsingStorageAccess,
+                           ContentParent* aSource) {
+  return CheckOnlyOwningProcessCanSet(aSource);
+}
+
+bool WindowContext::CanSet(FieldIndex<IDX_ShouldResistFingerprinting>,
+                           const bool& aShouldResistFingerprinting,
+                           ContentParent* aSource) {
+  return CheckOnlyOwningProcessCanSet(aSource);
+}
+
+bool WindowContext::CanSet(FieldIndex<IDX_OverriddenFingerprintingSettings>,
+                           const Maybe<RFPTargetSet>& aValue,
+                           ContentParent* aSource) {
+  return CheckOnlyOwningProcessCanSet(aSource);
+}
+
 bool WindowContext::CanSet(FieldIndex<IDX_IsSecureContext>,
                            const bool& aIsSecureContext,
                            ContentParent* aSource) {
@@ -265,11 +343,6 @@ bool WindowContext::CanSet(FieldIndex<IDX_IsLocalIP>, const bool& aValue,
   return CheckOnlyOwningProcessCanSet(aSource);
 }
 
-bool WindowContext::CanSet(FieldIndex<IDX_HadLazyLoadImage>, const bool& aValue,
-                           ContentParent* aSource) {
-  return IsTop();
-}
-
 bool WindowContext::CanSet(FieldIndex<IDX_AllowJavascript>, bool aValue,
                            ContentParent* aSource) {
   return (XRE_IsParentProcess() && !aSource) ||
@@ -330,20 +403,20 @@ void WindowContext::DidSet(FieldIndex<IDX_SHEntryHasUserInteraction>,
   }
 }
 
-void WindowContext::DidSet(FieldIndex<IDX_UserActivationState>) {
-  MOZ_ASSERT_IF(!IsInProcess(), mUserGestureStart.IsNull());
-  USER_ACTIVATION_LOG("Set user gesture activation %" PRIu8
+void WindowContext::DidSet(FieldIndex<IDX_UserActivationStateAndModifiers>) {
+  MOZ_ASSERT_IF(!IsInProcess(), mLastActivationTimestamp.IsNull());
+  USER_ACTIVATION_LOG("Set user gesture activation 0x%02" PRIu8
                       " for %s browsing context 0x%08" PRIx64,
-                      static_cast<uint8_t>(GetUserActivationState()),
+                      GetUserActivationStateAndModifiers(),
                       XRE_IsParentProcess() ? "Parent" : "Child", Id());
   if (IsInProcess()) {
     USER_ACTIVATION_LOG(
         "Set user gesture start time for %s browsing context 0x%08" PRIx64,
         XRE_IsParentProcess() ? "Parent" : "Child", Id());
     if (GetUserActivationState() == UserActivation::State::FullActivated) {
-      mUserGestureStart = TimeStamp::Now();
+      mLastActivationTimestamp = TimeStamp::Now();
     } else if (GetUserActivationState() == UserActivation::State::None) {
-      mUserGestureStart = TimeStamp();
+      mLastActivationTimestamp = TimeStamp();
     }
   }
 }
@@ -436,7 +509,8 @@ void WindowContext::AddSecurityState(uint32_t aStateFlags) {
                nsIWebProgressListener::STATE_BLOCKED_MIXED_DISPLAY_CONTENT |
                nsIWebProgressListener::STATE_BLOCKED_MIXED_ACTIVE_CONTENT |
                nsIWebProgressListener::STATE_HTTPS_ONLY_MODE_UPGRADED |
-               nsIWebProgressListener::STATE_HTTPS_ONLY_MODE_UPGRADE_FAILED)) ==
+               nsIWebProgressListener::STATE_HTTPS_ONLY_MODE_UPGRADE_FAILED |
+               nsIWebProgressListener::STATE_HTTPS_ONLY_MODE_UPGRADED_FIRST)) ==
                  aStateFlags,
              "Invalid flags specified!");
 
@@ -448,72 +522,144 @@ void WindowContext::AddSecurityState(uint32_t aStateFlags) {
   }
 }
 
-void WindowContext::NotifyUserGestureActivation() {
-  Unused << SetUserActivationState(UserActivation::State::FullActivated);
+void WindowContext::NotifyUserGestureActivation(
+    UserActivation::Modifiers
+        aModifiers /* = UserActivation::Modifiers::None() */) {
+  UserActivation::StateAndModifiers stateAndModifiers;
+  stateAndModifiers.SetState(UserActivation::State::FullActivated);
+  stateAndModifiers.SetModifiers(aModifiers);
+  if (auto* innerWindow = GetInnerWindow()) {
+    innerWindow->EnsureCloseWatcherManager()->NotifyUserInteraction();
+  }
+  Unused << SetUserActivationStateAndModifiers(stateAndModifiers.GetRawData());
 }
 
 void WindowContext::NotifyResetUserGestureActivation() {
-  Unused << SetUserActivationState(UserActivation::State::None);
+  UserActivation::StateAndModifiers stateAndModifiers;
+  stateAndModifiers.SetState(UserActivation::State::None);
+  Unused << SetUserActivationStateAndModifiers(stateAndModifiers.GetRawData());
 }
 
 bool WindowContext::HasBeenUserGestureActivated() {
   return GetUserActivationState() != UserActivation::State::None;
 }
 
-DOMHighResTimeStamp WindowContext::LastUserGestureTimeStamp() {
+const TimeStamp& WindowContext::GetUserGestureStart() const {
   MOZ_ASSERT(IsInProcess());
-  if (nsGlobalWindowInner* innerWindow = GetInnerWindow()) {
-    if (Performance* perf = innerWindow->GetPerformance()) {
-      return perf->GetDOMTiming()->TimeStampToDOMHighRes(mUserGestureStart);
-    }
-  }
-  NS_WARNING(
-      "Unable to calculate DOMHighResTimeStamp for LastUserGestureTimeStamp");
-  return 0;
+  return mLastActivationTimestamp;
 }
 
+// https://html.spec.whatwg.org/#transient-activation
 bool WindowContext::HasValidTransientUserGestureActivation() {
   MOZ_ASSERT(IsInProcess());
 
   if (GetUserActivationState() != UserActivation::State::FullActivated) {
-    // mUserGestureStart should be null if the document hasn't ever been
+    // mLastActivationTimestamp should be null if the document hasn't ever been
     // activated by user gesture
     MOZ_ASSERT_IF(GetUserActivationState() == UserActivation::State::None,
-                  mUserGestureStart.IsNull());
+                  mLastActivationTimestamp.IsNull());
     return false;
   }
 
-  MOZ_ASSERT(!mUserGestureStart.IsNull(),
-             "mUserGestureStart shouldn't be null if the document has ever "
-             "been activated by user gesture");
+  MOZ_ASSERT(
+      !mLastActivationTimestamp.IsNull(),
+      "mLastActivationTimestamp shouldn't be null if the document has ever "
+      "been activated by user gesture");
+
   TimeDuration timeout = TimeDuration::FromMilliseconds(
       StaticPrefs::dom_user_activation_transient_timeout());
 
+  // "When the current high resolution time given W is greater than or equal to
+  // the last activation timestamp in W, and less than the last activation
+  // timestamp in W plus the transient activation duration, then W is said to
+  // have transient activation."
   return timeout <= TimeDuration() ||
-         (TimeStamp::Now() - mUserGestureStart) <= timeout;
+         (TimeStamp::Now() - mLastActivationTimestamp) <= timeout;
 }
 
+// https://html.spec.whatwg.org/#consume-user-activation
 bool WindowContext::ConsumeTransientUserGestureActivation() {
   MOZ_ASSERT(IsInProcess());
   MOZ_ASSERT(IsCurrent());
 
+  // 1. If W's navigable is null, then return.
+
   if (!HasValidTransientUserGestureActivation()) {
     return false;
   }
 
+  // 2. Let top be W's navigable's top-level traversable.
   BrowsingContext* top = mBrowsingContext->Top();
+
+  // 3. Let navigables be the inclusive descendant navigables of top's active
+  // document.
   top->PreOrderWalk([&](BrowsingContext* aBrowsingContext) {
+    // 4. Let windows be the list of Window objects constructed by taking the
+    // active window of each item in navigables.
     WindowContext* windowContext = aBrowsingContext->GetCurrentWindowContext();
+
+    // 5. For each window in windows, if window's last activation timestamp is
+    // not positive infinity, then set window's last activation timestamp to
+    // negative infinity.
     if (windowContext && windowContext->GetUserActivationState() ==
                              UserActivation::State::FullActivated) {
-      Unused << windowContext->SetUserActivationState(
-          UserActivation::State::HasBeenActivated);
+      auto stateAndModifiers = UserActivation::StateAndModifiers(
+          GetUserActivationStateAndModifiers());
+      // Setting UserActivationStateAndModifiers will trigger
+      // DidSet(FieldIndex<IDX_UserActivationStateAndModifiers>),
+      // which in turn updates mLastActivationTimestamp.
+      stateAndModifiers.SetState(UserActivation::State::HasBeenActivated);
+      Unused << windowContext->SetUserActivationStateAndModifiers(
+          stateAndModifiers.GetRawData());
     }
   });
 
   return true;
 }
 
+// https://html.spec.whatwg.org/multipage/interaction.html#history-action-activation
+bool WindowContext::HasValidHistoryActivation() const {
+  MOZ_ASSERT(IsInProcess());
+  return mHistoryActivation != mLastActivationTimestamp;
+}
+
+// https://html.spec.whatwg.org/multipage/interaction.html#consume-history-action-user-activation
+// Step 1-2
+void WindowContext::ConsumeHistoryActivation() {
+  MOZ_ASSERT(IsInProcess());
+
+  // 1. If W's navigable is null, then return.
+
+  // 2. Let top be W's navigable's top-level traversable.
+  RefPtr<BrowsingContext> top = mBrowsingContext->Top();
+
+  // Consuming a history activation must happen across all child processes,
+  // including for example cross-origin iframes. As such we need to send an
+  // message over the IPC boundary to ensure out of processes contexts also
+  // consume their activations.
+  MOZ_ASSERT(XRE_IsContentProcess());
+  ContentChild::GetSingleton()->SendConsumeHistoryActivation(top);
+
+  // Update the local process children immediately.
+  top->ConsumeHistoryActivation();
+}
+
+void WindowContext::UpdateLastHistoryActivation() {
+  mHistoryActivation = mLastActivationTimestamp;
+}
+
+bool WindowContext::GetTransientUserGestureActivationModifiers(
+    UserActivation::Modifiers* aModifiers) {
+  if (!HasValidTransientUserGestureActivation()) {
+    return false;
+  }
+
+  auto stateAndModifiers =
+      UserActivation::StateAndModifiers(GetUserActivationStateAndModifiers());
+  *aModifiers = stateAndModifiers.GetModifiers();
+  return true;
+}
+
 bool WindowContext::CanShowPopup() {
   uint32_t permit = GetPopupPermission();
   if (permit == nsIPermissionManager::ALLOW_ACTION) {
@@ -526,6 +672,14 @@ bool WindowContext::CanShowPopup() {
   return !StaticPrefs::dom_disable_open_during_load();
 }
 
+void WindowContext::TransientSetHasActivePeerConnections() {
+  if (!IsTop()) {
+    return;
+  }
+
+  mFields.SetWithoutSyncing<IDX_HasActivePeerConnections>(true);
+}
+
 WindowContext::IPCInitializer WindowContext::GetIPCInitializer() {
   IPCInitializer init;
   init.mInnerWindowId = mInnerWindowId;
@@ -576,12 +730,16 @@ NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(WindowContext)
 
   NS_IMPL_CYCLE_COLLECTION_UNLINK(mBrowsingContext)
   NS_IMPL_CYCLE_COLLECTION_UNLINK(mChildren)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mNonSyntheticChildren)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mNonSyntheticLightDOMChildren);
   NS_IMPL_CYCLE_COLLECTION_UNLINK_PRESERVED_WRAPPER
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(WindowContext)
   NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mBrowsingContext)
   NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mChildren)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mNonSyntheticChildren)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mNonSyntheticLightDOMChildren)
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
 }  // namespace dom
