@@ -18,7 +18,7 @@
 #include "nsAtom.h"
 #include "nsReadableUtils.h"
 #include "nsUnicharUtils.h"
-#include "mozilla/LookAndFeel.h"
+#include "mozilla/StaticPrefs_ui.h"
 
 // Interfaces needed to be included
 #include "nsPresContext.h"
@@ -39,14 +39,15 @@
 #include "nsIRemoteTab.h"
 #include "nsIBrowserChild.h"
 #include "nsRect.h"
-#include "nsIWebBrowserChromeFocus.h"
 #include "nsIContent.h"
 #include "nsServiceManagerUtils.h"
 #include "nsViewManager.h"
 #include "nsView.h"
+#include "nsXULTooltipListener.h"
 #include "nsIConstraintValidation.h"
 #include "mozilla/Attributes.h"
 #include "mozilla/EventListenerManager.h"
+#include "mozilla/Try.h"
 #include "mozilla/dom/DragEvent.h"
 #include "mozilla/dom/Event.h"     // for Event
 #include "mozilla/dom/File.h"      // for input type=file
@@ -194,13 +195,6 @@ nsDocShellTreeOwner::GetInterface(const nsIID& aIID, void** aSink) {
     return NS_OK;
   }
 
-  if (aIID.Equals(NS_GET_IID(nsIWebBrowserChromeFocus))) {
-    if (mWebBrowserChromeWeak != nullptr) {
-      return mWebBrowserChromeWeak->QueryReferent(aIID, aSink);
-    }
-    return mOwnerWin->QueryInterface(aIID, aSink);
-  }
-
   if (aIID.Equals(NS_GET_IID(nsIPrompt))) {
     nsCOMPtr<nsIPrompt> prompt;
     EnsurePrompter();
@@ -453,16 +447,20 @@ nsDocShellTreeOwner::SizeShellTo(nsIDocShellTreeItem* aShellItem, int32_t aCX,
         do_QueryInterface(webBrowserChrome);
     if (browserChild) {
       // The XUL window to resize is in the parent process, but there we
-      // won't be able to get aShellItem to do the hack in
-      // AppWindow::SizeShellTo, so let's send the width and height of
-      // aShellItem too.
+      // won't be able to get the size of aShellItem. We can ask the parent
+      // process to change our size instead.
       nsCOMPtr<nsIBaseWindow> shellAsWin(do_QueryInterface(aShellItem));
       NS_ENSURE_TRUE(shellAsWin, NS_ERROR_FAILURE);
 
-      int32_t width = 0;
-      int32_t height = 0;
-      shellAsWin->GetSize(&width, &height);
-      return browserChild->RemoteSizeShellTo(aCX, aCY, width, height);
+      LayoutDeviceIntSize shellSize;
+      shellAsWin->GetSize(&shellSize.width, &shellSize.height);
+      LayoutDeviceIntSize deltaSize = LayoutDeviceIntSize(aCX, aCY) - shellSize;
+
+      LayoutDeviceIntSize currentSize;
+      GetSize(&currentSize.width, &currentSize.height);
+
+      LayoutDeviceIntSize newSize = currentSize + deltaSize;
+      return SetSize(newSize.width, newSize.height, true);
     }
     // XXX: this is weird, but we used to call a method here
     // (webBrowserChrome->SizeBrowserTo()) whose implementations all failed
@@ -470,33 +468,7 @@ nsDocShellTreeOwner::SizeShellTo(nsIDocShellTreeItem* aShellItem, int32_t aCX,
     return NS_ERROR_NOT_IMPLEMENTED;
   }
 
-  NS_ENSURE_TRUE(aShellItem, NS_ERROR_FAILURE);
-
-  RefPtr<Document> document = aShellItem->GetDocument();
-  NS_ENSURE_TRUE(document, NS_ERROR_FAILURE);
-
-  NS_ENSURE_TRUE(document->GetDocumentElement(), NS_ERROR_FAILURE);
-
-  // Set the preferred Size
-  // XXX
-  NS_ERROR("Implement this");
-  /*
-  Set the preferred size on the aShellItem.
-  */
-
-  RefPtr<nsPresContext> presContext = mWebBrowser->mDocShell->GetPresContext();
-  NS_ENSURE_TRUE(presContext, NS_ERROR_FAILURE);
-
-  RefPtr<PresShell> presShell = presContext->GetPresShell();
-  NS_ENSURE_TRUE(presShell, NS_ERROR_FAILURE);
-
-  NS_ENSURE_SUCCESS(
-      presShell->ResizeReflow(NS_UNCONSTRAINEDSIZE, NS_UNCONSTRAINEDSIZE),
-      NS_ERROR_FAILURE);
-
-  // XXX: this is weird, but we used to call a method here
-  // (webBrowserChrome->SizeBrowserTo()) whose implementations all failed like
-  // this, so...
+  MOZ_ASSERT_UNREACHABLE("This is unimplemented, API should be cleaned up");
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
@@ -512,16 +484,6 @@ nsDocShellTreeOwner::GetPersistence(bool* aPersistPosition, bool* aPersistSize,
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-NS_IMETHODIMP
-nsDocShellTreeOwner::GetTabCount(uint32_t* aResult) {
-  if (mTreeOwner) {
-    return mTreeOwner->GetTabCount(aResult);
-  }
-
-  *aResult = 0;
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 nsDocShellTreeOwner::GetHasPrimaryContent(bool* aResult) {
   *aResult = mPrimaryRemoteTab || mPrimaryContentShell;
@@ -533,8 +495,7 @@ nsDocShellTreeOwner::GetHasPrimaryContent(bool* aResult) {
 //*****************************************************************************
 
 NS_IMETHODIMP
-nsDocShellTreeOwner::InitWindow(nativeWindow aParentNativeWindow,
-                                nsIWidget* aParentWidget, int32_t aX,
+nsDocShellTreeOwner::InitWindow(nsIWidget* aParentWidget, int32_t aX,
                                 int32_t aY, int32_t aCX, int32_t aCY) {
   return NS_ERROR_NULL_POINTER;
 }
@@ -552,14 +513,8 @@ nsDocShellTreeOwner::Destroy() {
   return NS_ERROR_NULL_POINTER;
 }
 
-NS_IMETHODIMP
-nsDocShellTreeOwner::GetUnscaledDevicePixelsPerCSSPixel(double* aScale) {
-  if (mWebBrowser) {
-    return mWebBrowser->GetUnscaledDevicePixelsPerCSSPixel(aScale);
-  }
-
-  *aScale = 1.0;
-  return NS_OK;
+double nsDocShellTreeOwner::GetWidgetCSSToDeviceScale() {
+  return mWebBrowser ? mWebBrowser->GetWidgetCSSToDeviceScale() : 1.0;
 }
 
 NS_IMETHODIMP
@@ -586,68 +541,62 @@ nsDocShellTreeOwner::SetPositionDesktopPix(int32_t aX, int32_t aY) {
 
 NS_IMETHODIMP
 nsDocShellTreeOwner::SetPosition(int32_t aX, int32_t aY) {
-  nsCOMPtr<nsIEmbeddingSiteWindow> ownerWin = GetOwnerWin();
-  if (ownerWin) {
-    return ownerWin->SetDimensions(nsIEmbeddingSiteWindow::DIM_FLAGS_POSITION,
-                                   aX, aY, 0, 0);
-  }
-  return NS_ERROR_NULL_POINTER;
+  return SetDimensions(
+      {DimensionKind::Outer, Some(aX), Some(aY), Nothing(), Nothing()});
 }
 
 NS_IMETHODIMP
 nsDocShellTreeOwner::GetPosition(int32_t* aX, int32_t* aY) {
-  nsCOMPtr<nsIEmbeddingSiteWindow> ownerWin = GetOwnerWin();
-  if (ownerWin) {
-    return ownerWin->GetDimensions(nsIEmbeddingSiteWindow::DIM_FLAGS_POSITION,
-                                   aX, aY, nullptr, nullptr);
-  }
-  return NS_ERROR_NULL_POINTER;
+  return GetDimensions(DimensionKind::Outer, aX, aY, nullptr, nullptr);
 }
 
 NS_IMETHODIMP
 nsDocShellTreeOwner::SetSize(int32_t aCX, int32_t aCY, bool aRepaint) {
-  nsCOMPtr<nsIEmbeddingSiteWindow> ownerWin = GetOwnerWin();
-  if (ownerWin) {
-    return ownerWin->SetDimensions(nsIEmbeddingSiteWindow::DIM_FLAGS_SIZE_OUTER,
-                                   0, 0, aCX, aCY);
-  }
-  return NS_ERROR_NULL_POINTER;
+  return SetDimensions(
+      {DimensionKind::Outer, Nothing(), Nothing(), Some(aCX), Some(aCY)});
 }
 
 NS_IMETHODIMP
 nsDocShellTreeOwner::GetSize(int32_t* aCX, int32_t* aCY) {
-  nsCOMPtr<nsIEmbeddingSiteWindow> ownerWin = GetOwnerWin();
-  if (ownerWin) {
-    return ownerWin->GetDimensions(nsIEmbeddingSiteWindow::DIM_FLAGS_SIZE_OUTER,
-                                   nullptr, nullptr, aCX, aCY);
-  }
-  return NS_ERROR_NULL_POINTER;
+  return GetDimensions(DimensionKind::Outer, nullptr, nullptr, aCX, aCY);
 }
 
 NS_IMETHODIMP
 nsDocShellTreeOwner::SetPositionAndSize(int32_t aX, int32_t aY, int32_t aCX,
                                         int32_t aCY, uint32_t aFlags) {
-  nsCOMPtr<nsIEmbeddingSiteWindow> ownerWin = GetOwnerWin();
-  if (ownerWin) {
-    return ownerWin->SetDimensions(
-        nsIEmbeddingSiteWindow::DIM_FLAGS_SIZE_OUTER |
-            nsIEmbeddingSiteWindow::DIM_FLAGS_POSITION,
-        aX, aY, aCX, aCY);
-  }
-  return NS_ERROR_NULL_POINTER;
+  return SetDimensions(
+      {DimensionKind::Outer, Some(aX), Some(aY), Some(aCX), Some(aCY)});
 }
 
 NS_IMETHODIMP
 nsDocShellTreeOwner::GetPositionAndSize(int32_t* aX, int32_t* aY, int32_t* aCX,
                                         int32_t* aCY) {
-  nsCOMPtr<nsIEmbeddingSiteWindow> ownerWin = GetOwnerWin();
+  return GetDimensions(DimensionKind::Outer, aX, aY, aCX, aCY);
+}
+
+NS_IMETHODIMP
+nsDocShellTreeOwner::SetDimensions(DimensionRequest&& aRequest) {
+  nsCOMPtr<nsIBaseWindow> ownerWin = GetOwnerWin();
   if (ownerWin) {
-    return ownerWin->GetDimensions(
-        nsIEmbeddingSiteWindow::DIM_FLAGS_SIZE_OUTER |
-            nsIEmbeddingSiteWindow::DIM_FLAGS_POSITION,
-        aX, aY, aCX, aCY);
+    return ownerWin->SetDimensions(std::move(aRequest));
   }
-  return NS_ERROR_NULL_POINTER;
+
+  nsCOMPtr<nsIWebBrowserChrome> webBrowserChrome = GetWebBrowserChrome();
+  NS_ENSURE_STATE(webBrowserChrome);
+  return webBrowserChrome->SetDimensions(std::move(aRequest));
+}
+
+NS_IMETHODIMP
+nsDocShellTreeOwner::GetDimensions(DimensionKind aDimensionKind, int32_t* aX,
+                                   int32_t* aY, int32_t* aCX, int32_t* aCY) {
+  nsCOMPtr<nsIBaseWindow> ownerWin = GetOwnerWin();
+  if (ownerWin) {
+    return ownerWin->GetDimensions(aDimensionKind, aX, aY, aCX, aCY);
+  }
+
+  nsCOMPtr<nsIWebBrowserChrome> webBrowserChrome = GetWebBrowserChrome();
+  NS_ENSURE_STATE(webBrowserChrome);
+  return webBrowserChrome->GetDimensions(aDimensionKind, aX, aY, aCX, aCY);
 }
 
 NS_IMETHODIMP
@@ -663,20 +612,6 @@ nsDocShellTreeOwner::SetParentWidget(nsIWidget* aParentWidget) {
   return NS_ERROR_NULL_POINTER;
 }
 
-NS_IMETHODIMP
-nsDocShellTreeOwner::GetParentNativeWindow(nativeWindow* aParentNativeWindow) {
-  nsCOMPtr<nsIEmbeddingSiteWindow> ownerWin = GetOwnerWin();
-  if (ownerWin) {
-    return ownerWin->GetSiteWindow(aParentNativeWindow);
-  }
-  return NS_ERROR_NULL_POINTER;
-}
-
-NS_IMETHODIMP
-nsDocShellTreeOwner::SetParentNativeWindow(nativeWindow aParentNativeWindow) {
-  return NS_ERROR_NULL_POINTER;
-}
-
 NS_IMETHODIMP
 nsDocShellTreeOwner::GetNativeHandle(nsAString& aNativeHandle) {
   // the nativeHandle should be accessed from nsIAppWindow
@@ -685,16 +620,17 @@ nsDocShellTreeOwner::GetNativeHandle(nsAString& aNativeHandle) {
 
 NS_IMETHODIMP
 nsDocShellTreeOwner::GetVisibility(bool* aVisibility) {
-  nsCOMPtr<nsIEmbeddingSiteWindow> ownerWin = GetOwnerWin();
+  nsCOMPtr<nsIBaseWindow> ownerWin = GetOwnerWin();
   if (ownerWin) {
     return ownerWin->GetVisibility(aVisibility);
   }
-  return NS_ERROR_NULL_POINTER;
+
+  return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
 nsDocShellTreeOwner::SetVisibility(bool aVisibility) {
-  nsCOMPtr<nsIEmbeddingSiteWindow> ownerWin = GetOwnerWin();
+  nsCOMPtr<nsIBaseWindow> ownerWin = GetOwnerWin();
   if (ownerWin) {
     return ownerWin->SetVisibility(aVisibility);
   }
@@ -720,7 +656,7 @@ nsDocShellTreeOwner::GetMainWidget(nsIWidget** aMainWidget) {
 
 NS_IMETHODIMP
 nsDocShellTreeOwner::GetTitle(nsAString& aTitle) {
-  nsCOMPtr<nsIEmbeddingSiteWindow> ownerWin = GetOwnerWin();
+  nsCOMPtr<nsIBaseWindow> ownerWin = GetOwnerWin();
   if (ownerWin) {
     return ownerWin->GetTitle(aTitle);
   }
@@ -729,7 +665,7 @@ nsDocShellTreeOwner::GetTitle(nsAString& aTitle) {
 
 NS_IMETHODIMP
 nsDocShellTreeOwner::SetTitle(const nsAString& aTitle) {
-  nsCOMPtr<nsIEmbeddingSiteWindow> ownerWin = GetOwnerWin();
+  nsCOMPtr<nsIBaseWindow> ownerWin = GetOwnerWin();
   if (ownerWin) {
     return ownerWin->SetTitle(aTitle);
   }
@@ -851,8 +787,7 @@ nsDocShellTreeOwner::SetWebBrowserChrome(
     if (supportsweak) {
       supportsweak->GetWeakReference(getter_AddRefs(mWebBrowserChromeWeak));
     } else {
-      nsCOMPtr<nsIEmbeddingSiteWindow> ownerWin(
-          do_QueryInterface(aWebBrowserChrome));
+      nsCOMPtr<nsIBaseWindow> ownerWin(do_QueryInterface(aWebBrowserChrome));
       nsCOMPtr<nsIInterfaceRequestor> requestor(
           do_QueryInterface(aWebBrowserChrome));
 
@@ -959,7 +894,8 @@ nsDocShellTreeOwner::HandleEvent(Event* aEvent) {
       aEvent->PreventDefault();
     }
   } else if (eventType.EqualsLiteral("drop")) {
-    nsIWebNavigation* webnav = static_cast<nsIWebNavigation*>(mWebBrowser);
+    nsCOMPtr<nsIWebNavigation> webnav =
+        static_cast<nsIWebNavigation*>(mWebBrowser);
 
     // The page might have cancelled the dragover event itself, so check to
     // make sure that the link can be dropped first.
@@ -997,9 +933,9 @@ nsDocShellTreeOwner::HandleEvent(Event* aEvent) {
               LoadURIOptions loadURIOptions;
               loadURIOptions.mTriggeringPrincipal = triggeringPrincipal;
               nsCOMPtr<nsIContentSecurityPolicy> csp;
-              handler->GetCSP(dragEvent, getter_AddRefs(csp));
+              handler->GetCsp(dragEvent, getter_AddRefs(csp));
               loadURIOptions.mCsp = csp;
-              webnav->LoadURI(url, loadURIOptions);
+              webnav->FixupAndLoadURIString(url, loadURIOptions);
             }
           }
         }
@@ -1024,8 +960,8 @@ nsDocShellTreeOwner::GetWebBrowserChrome() {
   return chrome.forget();
 }
 
-already_AddRefed<nsIEmbeddingSiteWindow> nsDocShellTreeOwner::GetOwnerWin() {
-  nsCOMPtr<nsIEmbeddingSiteWindow> win;
+already_AddRefed<nsIBaseWindow> nsDocShellTreeOwner::GetOwnerWin() {
+  nsCOMPtr<nsIBaseWindow> win;
   if (mWebBrowserChromeWeak) {
     win = do_QueryReferent(mWebBrowserChromeWeak);
   } else if (mOwnerWin) {
@@ -1100,21 +1036,14 @@ ChromeTooltipListener::AddChromeListeners() {
 NS_IMETHODIMP
 ChromeTooltipListener::AddTooltipListener() {
   if (mEventTarget) {
-    nsresult rv = NS_OK;
-#ifndef XP_WIN
-    rv =
-        mEventTarget->AddSystemEventListener(u"keydown"_ns, this, false, false);
-    NS_ENSURE_SUCCESS(rv, rv);
-#endif
-    rv = mEventTarget->AddSystemEventListener(u"mousedown"_ns, this, false,
-                                              false);
-    NS_ENSURE_SUCCESS(rv, rv);
-    rv = mEventTarget->AddSystemEventListener(u"mouseout"_ns, this, false,
-                                              false);
-    NS_ENSURE_SUCCESS(rv, rv);
-    rv = mEventTarget->AddSystemEventListener(u"mousemove"_ns, this, false,
-                                              false);
-    NS_ENSURE_SUCCESS(rv, rv);
+    MOZ_TRY(mEventTarget->AddSystemEventListener(u"keydown"_ns, this, false,
+                                                 false));
+    MOZ_TRY(mEventTarget->AddSystemEventListener(u"mousedown"_ns, this, false,
+                                                 false));
+    MOZ_TRY(mEventTarget->AddSystemEventListener(u"mouseout"_ns, this, false,
+                                                 false));
+    MOZ_TRY(mEventTarget->AddSystemEventListener(u"mousemove"_ns, this, false,
+                                                 false));
 
     mTooltipListenerInstalled = true;
   }
@@ -1141,9 +1070,7 @@ ChromeTooltipListener::RemoveChromeListeners() {
 NS_IMETHODIMP
 ChromeTooltipListener::RemoveTooltipListener() {
   if (mEventTarget) {
-#ifndef XP_WIN
     mEventTarget->RemoveSystemEventListener(u"keydown"_ns, this, false);
-#endif
     mEventTarget->RemoveSystemEventListener(u"mousedown"_ns, this, false);
     mEventTarget->RemoveSystemEventListener(u"mouseout"_ns, this, false);
     mEventTarget->RemoveSystemEventListener(u"mousemove"_ns, this, false);
@@ -1162,10 +1089,9 @@ ChromeTooltipListener::HandleEvent(Event* aEvent) {
     return HideTooltip();
   } else if (eventType.EqualsLiteral("keydown")) {
     WidgetKeyboardEvent* keyEvent = aEvent->WidgetEventPtr()->AsKeyboardEvent();
-    if (!keyEvent->IsModifierKeyEvent()) {
+    if (nsXULTooltipListener::KeyEventHidesTooltip(*keyEvent)) {
       return HideTooltip();
     }
-
     return NS_OK;
   } else if (eventType.EqualsLiteral("mouseout")) {
     // Reset flag so that tooltip will display on the next MouseMove
@@ -1182,6 +1108,10 @@ ChromeTooltipListener::HandleEvent(Event* aEvent) {
 // If we're a tooltip, fire off a timer to see if a tooltip should be shown. If
 // the timer fires, we cache the node in |mPossibleTooltipNode|.
 nsresult ChromeTooltipListener::MouseMove(Event* aMouseEvent) {
+  if (!nsXULTooltipListener::ShowTooltips()) {
+    return NS_OK;
+  }
+
   MouseEvent* mouseEvent = aMouseEvent->AsMouseEvent();
   if (!mouseEvent) {
     return NS_OK;
@@ -1191,7 +1121,8 @@ nsresult ChromeTooltipListener::MouseMove(Event* aMouseEvent) {
   // within the timer callback. On win32, we'll get a MouseMove event even when
   // a popup goes away -- even when the mouse doesn't change position! To get
   // around this, we make sure the mouse has really moved before proceeding.
-  CSSIntPoint newMouseClientPoint = mouseEvent->ClientPoint();
+  const CSSIntPoint newMouseClientPoint =
+      RoundedToInt(mouseEvent->ClientPoint());
   if (mMouseClientPoint == newMouseClientPoint) {
     return NS_OK;
   }
@@ -1214,20 +1145,15 @@ nsresult ChromeTooltipListener::MouseMove(Event* aMouseEvent) {
   }
 
   if (!mShowingTooltip) {
-    nsIEventTarget* target = nullptr;
-    if (nsCOMPtr<EventTarget> eventTarget = aMouseEvent->GetComposedTarget()) {
+    if (nsCOMPtr<EventTarget> eventTarget = aMouseEvent->GetOriginalTarget()) {
       mPossibleTooltipNode = nsINode::FromEventTarget(eventTarget);
-      nsCOMPtr<nsIGlobalObject> global(eventTarget->GetOwnerGlobal());
-      if (global) {
-        target = global->EventTargetFor(TaskCategory::UI);
-      }
     }
 
     if (mPossibleTooltipNode) {
       nsresult rv = NS_NewTimerWithFuncCallback(
           getter_AddRefs(mTooltipTimer), sTooltipCallback, this,
-          LookAndFeel::GetInt(LookAndFeel::IntID::TooltipDelay, 500),
-          nsITimer::TYPE_ONE_SHOT, "ChromeTooltipListener::MouseMove", target);
+          StaticPrefs::ui_tooltip_delay_ms(), nsITimer::TYPE_ONE_SHOT,
+          "ChromeTooltipListener::MouseMove", GetMainThreadSerialEventTarget());
       if (NS_FAILED(rv)) {
         mPossibleTooltipNode = nullptr;
         NS_WARNING("Could not create a timer for tooltip tracking");
