@@ -6,19 +6,23 @@
 
 #include <math.h>
 
+#include "CSSEditUtils.h"
+#include "EditAction.h"
+#include "EditorLineBreak.h"
+#include "HTMLEditHelpers.h"
 #include "HTMLEditorEventListener.h"
 #include "HTMLEditUtils.h"
-#include "mozilla/EditAction.h"
+
 #include "mozilla/EventListenerManager.h"
+#include "mozilla/mozalloc.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/PresShell.h"
+#include "mozilla/StaticPrefs_editor.h"
 #include "mozilla/dom/AncestorIterator.h"
 #include "mozilla/dom/Selection.h"
 #include "mozilla/dom/Element.h"
 #include "mozilla/dom/EventTarget.h"
-#include "mozilla/mozalloc.h"
 #include "nsAString.h"
-#include "nsAlgorithm.h"
 #include "nsCOMPtr.h"
 #include "nsComputedDOMStyle.h"
 #include "nsDebug.h"
@@ -44,8 +48,6 @@ using namespace dom;
 
 nsresult HTMLEditor::SetSelectionToAbsoluteOrStaticAsAction(
     bool aEnabled, nsIPrincipal* aPrincipal) {
-  MOZ_ASSERT(IsEditActionDataAvailable());
-
   AutoEditActionDataSetter editActionData(
       *this, EditAction::eSetPositionToAbsoluteOrStatic, aPrincipal);
   nsresult rv = editActionData.CanHandleAndMaybeDispatchBeforeInputEvent();
@@ -55,17 +57,26 @@ nsresult HTMLEditor::SetSelectionToAbsoluteOrStaticAsAction(
     return rv;
   }
 
-  if (aEnabled) {
-    EditActionResult result = SetSelectionToAbsoluteAsSubAction();
-    NS_WARNING_ASSERTION(
-        result.Succeeded(),
-        "HTMLEditor::SetSelectionToAbsoluteAsSubAction() failed");
-    return result.Rv();
+  const RefPtr<Element> editingHost = ComputeEditingHost();
+  if (!editingHost) {
+    return NS_SUCCESS_DOM_NO_OPERATION;
   }
-  EditActionResult result = SetSelectionToStaticAsSubAction();
-  NS_WARNING_ASSERTION(result.Succeeded(),
-                       "HTMLEditor::SetSelectionToStaticAsSubAction() failed");
-  return result.Rv();
+
+  if (aEnabled) {
+    Result<EditActionResult, nsresult> result =
+        SetSelectionToAbsoluteAsSubAction(*editingHost);
+    if (MOZ_UNLIKELY(result.isErr())) {
+      NS_WARNING("HTMLEditor::SetSelectionToAbsoluteAsSubAction() failed");
+      return result.unwrapErr();
+    }
+    return NS_OK;
+  }
+  Result<EditActionResult, nsresult> result = SetSelectionToStaticAsSubAction();
+  if (MOZ_UNLIKELY(result.isErr())) {
+    NS_WARNING("HTMLEditor::SetSelectionToStaticAsSubAction() failed");
+    return result.unwrapErr();
+  }
+  return NS_OK;
 }
 
 already_AddRefed<Element>
@@ -124,6 +135,12 @@ NS_IMETHODIMP HTMLEditor::SetAbsolutePositioningEnabled(bool aIsEnabled) {
   return NS_OK;
 }
 
+NS_IMETHODIMP HTMLEditor::GetIsAbsolutePositioningActive(bool* aIsActive) {
+  MOZ_ASSERT(aIsActive);
+  *aIsActive = !!mAbsolutelyPositionedObject;
+  return NS_OK;
+}
+
 Result<int32_t, nsresult> HTMLEditor::AddZIndexWithTransaction(
     nsStyledElement& aStyledElement, int32_t aChange) {
   if (!aChange) {
@@ -151,8 +168,8 @@ nsresult HTMLEditor::SetZIndexWithTransaction(nsStyledElement& aStyledElement,
   nsAutoString zIndexValue;
   zIndexValue.AppendInt(aZIndex);
 
-  nsresult rv = mCSSEditUtils->SetCSSPropertyWithTransaction(
-      aStyledElement, *nsGkAtoms::z_index, zIndexValue);
+  nsresult rv = CSSEditUtils::SetCSSPropertyWithTransaction(
+      *this, aStyledElement, *nsGkAtoms::z_index, zIndexValue);
   if (rv == NS_ERROR_EDITOR_DESTROYED) {
     NS_WARNING(
         "CSSEditUtils::SetCSSPropertyWithTransaction(nsGkAtoms::z_index) "
@@ -178,10 +195,12 @@ nsresult HTMLEditor::AddZIndexAsAction(int32_t aChange,
     return EditorBase::ToGenericNSResult(rv);
   }
 
-  EditActionResult result = AddZIndexAsSubAction(aChange);
-  NS_WARNING_ASSERTION(result.Succeeded(),
-                       "HTMLEditor::AddZIndexAsSubAction() failed");
-  return EditorBase::ToGenericNSResult(result.Rv());
+  Result<EditActionResult, nsresult> result = AddZIndexAsSubAction(aChange);
+  if (MOZ_UNLIKELY(result.isErr())) {
+    NS_WARNING("HTMLEditor::AddZIndexAsSubAction() failed");
+    return EditorBase::ToGenericNSResult(result.unwrapErr());
+  }
+  return NS_OK;
 }
 
 int32_t HTMLEditor::GetZIndex(Element& aElement) {
@@ -372,7 +391,9 @@ void HTMLEditor::HideGrabberInternal() {
 nsresult HTMLEditor::ShowGrabberInternal(Element& aElement) {
   MOZ_ASSERT(IsEditActionDataAvailable());
 
-  if (NS_WARN_IF(!IsDescendantOfEditorRoot(&aElement))) {
+  const RefPtr<Element> editingHost = ComputeEditingHost();
+  if (NS_WARN_IF(!editingHost) ||
+      NS_WARN_IF(!aElement.IsInclusiveDescendantOf(editingHost))) {
     return NS_ERROR_UNEXPECTED;
   }
 
@@ -459,7 +480,7 @@ nsresult HTMLEditor::StartMoving() {
   if (RefPtr<nsStyledElement> positioningShadowStyledElement =
           nsStyledElement::FromNode(mPositioningShadow.get())) {
     nsresult rv;
-    rv = mCSSEditUtils->SetCSSPropertyPixelsWithoutTransaction(
+    rv = CSSEditUtils::SetCSSPropertyPixelsWithoutTransaction(
         *positioningShadowStyledElement, *nsGkAtoms::width,
         mPositionedObjectWidth);
     if (rv == NS_ERROR_EDITOR_DESTROYED) {
@@ -471,7 +492,7 @@ nsresult HTMLEditor::StartMoving() {
     NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
                          "CSSEditUtils::SetCSSPropertyPixelsWithoutTransaction("
                          "nsGkAtoms::width) failed, but ignored");
-    rv = mCSSEditUtils->SetCSSPropertyPixelsWithoutTransaction(
+    rv = CSSEditUtils::SetCSSPropertyPixelsWithoutTransaction(
         *positioningShadowStyledElement, *nsGkAtoms::height,
         mPositionedObjectHeight);
     if (rv == NS_ERROR_EDITOR_DESTROYED) {
@@ -489,7 +510,7 @@ nsresult HTMLEditor::StartMoving() {
   return NS_OK;  // XXX Looks like nobody refers this result
 }
 
-void HTMLEditor::SnapToGrid(int32_t& newX, int32_t& newY) {
+void HTMLEditor::SnapToGrid(int32_t& newX, int32_t& newY) const {
   if (mSnapToGridEnabled && mGridSize) {
     newX = (int32_t)floor(((float)newX / (float)mGridSize) + 0.5f) * mGridSize;
     newY = (int32_t)floor(((float)newY / (float)mGridSize) + 0.5f) * mGridSize;
@@ -565,8 +586,8 @@ nsresult HTMLEditor::SetFinalPosition(int32_t aX, int32_t aY) {
   y.AppendInt(newY);
 
   // we want one transaction only from a user's point of view
-  AutoPlaceholderBatch treatAsOneTransaction(*this,
-                                             ScrollSelectionIntoView::Yes);
+  AutoPlaceholderBatch treatAsOneTransaction(
+      *this, ScrollSelectionIntoView::Yes, __FUNCTION__);
 
   if (NS_WARN_IF(!mAbsolutelyPositionedObject)) {
     return NS_ERROR_FAILURE;
@@ -574,8 +595,8 @@ nsresult HTMLEditor::SetFinalPosition(int32_t aX, int32_t aY) {
   if (RefPtr<nsStyledElement> styledAbsolutelyPositionedElement =
           nsStyledElement::FromNode(mAbsolutelyPositionedObject)) {
     nsresult rv;
-    rv = mCSSEditUtils->SetCSSPropertyPixelsWithTransaction(
-        *styledAbsolutelyPositionedElement, *nsGkAtoms::top, newY);
+    rv = CSSEditUtils::SetCSSPropertyPixelsWithTransaction(
+        *this, *styledAbsolutelyPositionedElement, *nsGkAtoms::top, newY);
     if (rv == NS_ERROR_EDITOR_DESTROYED) {
       NS_WARNING(
           "CSSEditUtils::SetCSSPropertyPixelsWithTransaction(nsGkAtoms::top) "
@@ -586,8 +607,8 @@ nsresult HTMLEditor::SetFinalPosition(int32_t aX, int32_t aY) {
         NS_SUCCEEDED(rv),
         "CSSEditUtils::SetCSSPropertyPixelsWithTransaction(nsGkAtoms::top) "
         "failed, but ignored");
-    rv = mCSSEditUtils->SetCSSPropertyPixelsWithTransaction(
-        *styledAbsolutelyPositionedElement, *nsGkAtoms::left, newX);
+    rv = CSSEditUtils::SetCSSPropertyPixelsWithTransaction(
+        *this, *styledAbsolutelyPositionedElement, *nsGkAtoms::left, newX);
     if (rv == NS_ERROR_EDITOR_DESTROYED) {
       NS_WARNING(
           "CSSEditUtils::SetCSSPropertyPixelsWithTransaction(nsGkAtoms::left) "
@@ -609,15 +630,6 @@ nsresult HTMLEditor::SetFinalPosition(int32_t aX, int32_t aY) {
   return rv;
 }
 
-void HTMLEditor::AddPositioningOffset(int32_t& aX, int32_t& aY) {
-  // Get the positioning offset
-  int32_t positioningOffset =
-      Preferences::GetInt("editor.positioning.offset", 0);
-
-  aX += positioningOffset;
-  aY += positioningOffset;
-}
-
 nsresult HTMLEditor::SetPositionToAbsoluteOrStatic(Element& aElement,
                                                    bool aEnabled) {
   nsAutoString positionValue;
@@ -647,8 +659,8 @@ nsresult HTMLEditor::SetPositionToAbsoluteOrStatic(Element& aElement,
 nsresult HTMLEditor::SetPositionToAbsolute(Element& aElement) {
   MOZ_ASSERT(IsEditActionDataAvailable());
 
-  AutoPlaceholderBatch treatAsOneTransaction(*this,
-                                             ScrollSelectionIntoView::Yes);
+  AutoPlaceholderBatch treatAsOneTransaction(
+      *this, ScrollSelectionIntoView::Yes, __FUNCTION__);
 
   int32_t x, y;
   DebugOnly<nsresult> rvIgnored = GetElementOrigin(aElement, x, y);
@@ -659,8 +671,9 @@ nsresult HTMLEditor::SetPositionToAbsolute(Element& aElement) {
   if (styledElement) {
     // MOZ_KnownLive(*styledElement): aElement's lifetime must be guarantted
     // by the caller because of MOZ_CAN_RUN_SCRIPT method.
-    nsresult rv = mCSSEditUtils->SetCSSPropertyWithTransaction(
-        MOZ_KnownLive(*styledElement), *nsGkAtoms::position, u"absolute"_ns);
+    nsresult rv = CSSEditUtils::SetCSSPropertyWithTransaction(
+        *this, MOZ_KnownLive(*styledElement), *nsGkAtoms::position,
+        u"absolute"_ns);
     if (rv == NS_ERROR_EDITOR_DESTROYED) {
       NS_WARNING(
           "CSSEditUtils::SetCSSProperyWithTransaction(nsGkAtoms::Position) "
@@ -673,7 +686,6 @@ nsresult HTMLEditor::SetPositionToAbsolute(Element& aElement) {
         "absolute) failed, but ignored");
   }
 
-  AddPositioningOffset(x, y);
   SnapToGrid(x, y);
   if (styledElement) {
     // MOZ_KnownLive(*styledElement): aElement's lifetime must be guarantted
@@ -692,14 +704,25 @@ nsresult HTMLEditor::SetPositionToAbsolute(Element& aElement) {
   if (parentNode->GetChildCount() != 1) {
     return NS_OK;
   }
-  Result<RefPtr<Element>, nsresult> resultOfInsertingBRElement =
-      InsertBRElementWithTransaction(EditorDOMPoint(parentNode, 0));
-  if (resultOfInsertingBRElement.isErr()) {
-    NS_WARNING("HTMLEditor::InsertBRElementWithTransaction() failed");
-    return resultOfInsertingBRElement.unwrapErr();
+  Result<CreateLineBreakResult, nsresult> insertBRElementResultOrError =
+      InsertLineBreak(WithTransaction::Yes, LineBreakType::BRElement,
+                      EditorDOMPoint(parentNode, 0u));
+  if (MOZ_UNLIKELY(insertBRElementResultOrError.isErr())) {
+    NS_WARNING(
+        "HTMLEditor::InsertLineBreak(WithTransaction::Yes, "
+        "LineBreakType::BRElement) failed");
+    return insertBRElementResultOrError.unwrapErr();
   }
-  MOZ_ASSERT(resultOfInsertingBRElement.inspect());
-  return NS_OK;
+  CreateLineBreakResult insertBRElementResult =
+      insertBRElementResultOrError.unwrap();
+  MOZ_ASSERT(insertBRElementResult.Handled());
+  // XXX Is this intentional selection change?
+  nsresult rv = insertBRElementResult.SuggestCaretPointTo(
+      *this, {SuggestCaret::OnlyIfHasSuggestion,
+              SuggestCaret::OnlyIfTransactionsAllowedToDoIt});
+  NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
+                       "CreateElementResult::SuggestCaretPointTo() failed");
+  return rv;
 }
 
 nsresult HTMLEditor::SetPositionToStatic(Element& aElement) {
@@ -708,14 +731,14 @@ nsresult HTMLEditor::SetPositionToStatic(Element& aElement) {
     return NS_ERROR_INVALID_ARG;
   }
 
-  AutoPlaceholderBatch treatAsOneTransaction(*this,
-                                             ScrollSelectionIntoView::Yes);
+  AutoPlaceholderBatch treatAsOneTransaction(
+      *this, ScrollSelectionIntoView::Yes, __FUNCTION__);
 
   nsresult rv;
   // MOZ_KnownLive(*styledElement): aElement's lifetime must be guarantted
   // by the caller because of MOZ_CAN_RUN_SCRIPT method.
-  rv = mCSSEditUtils->RemoveCSSPropertyWithTransaction(
-      MOZ_KnownLive(*styledElement), *nsGkAtoms::position, u""_ns);
+  rv = CSSEditUtils::RemoveCSSPropertyWithTransaction(
+      *this, MOZ_KnownLive(*styledElement), *nsGkAtoms::position, u""_ns);
   if (rv == NS_ERROR_EDITOR_DESTROYED) {
     NS_WARNING(
         "CSSEditUtils::RemoveCSSPropertyWithTransaction(nsGkAtoms::position) "
@@ -728,8 +751,8 @@ nsresult HTMLEditor::SetPositionToStatic(Element& aElement) {
       "failed, but ignored");
   // MOZ_KnownLive(*styledElement): aElement's lifetime must be guarantted
   // by the caller because of MOZ_CAN_RUN_SCRIPT method.
-  rv = mCSSEditUtils->RemoveCSSPropertyWithTransaction(
-      MOZ_KnownLive(*styledElement), *nsGkAtoms::top, u""_ns);
+  rv = CSSEditUtils::RemoveCSSPropertyWithTransaction(
+      *this, MOZ_KnownLive(*styledElement), *nsGkAtoms::top, u""_ns);
   if (rv == NS_ERROR_EDITOR_DESTROYED) {
     NS_WARNING(
         "CSSEditUtils::RemoveCSSPropertyWithTransaction(nsGkAtoms::top) "
@@ -742,8 +765,8 @@ nsresult HTMLEditor::SetPositionToStatic(Element& aElement) {
       "failed, but ignored");
   // MOZ_KnownLive(*styledElement): aElement's lifetime must be guarantted
   // by the caller because of MOZ_CAN_RUN_SCRIPT method.
-  rv = mCSSEditUtils->RemoveCSSPropertyWithTransaction(
-      MOZ_KnownLive(*styledElement), *nsGkAtoms::left, u""_ns);
+  rv = CSSEditUtils::RemoveCSSPropertyWithTransaction(
+      *this, MOZ_KnownLive(*styledElement), *nsGkAtoms::left, u""_ns);
   if (rv == NS_ERROR_EDITOR_DESTROYED) {
     NS_WARNING(
         "CSSEditUtils::RemoveCSSPropertyWithTransaction(nsGkAtoms::left) "
@@ -756,8 +779,8 @@ nsresult HTMLEditor::SetPositionToStatic(Element& aElement) {
       "failed, but ignored");
   // MOZ_KnownLive(*styledElement): aElement's lifetime must be guarantted
   // by the caller because of MOZ_CAN_RUN_SCRIPT method.
-  rv = mCSSEditUtils->RemoveCSSPropertyWithTransaction(
-      MOZ_KnownLive(*styledElement), *nsGkAtoms::z_index, u""_ns);
+  rv = CSSEditUtils::RemoveCSSPropertyWithTransaction(
+      *this, MOZ_KnownLive(*styledElement), *nsGkAtoms::z_index, u""_ns);
   if (rv == NS_ERROR_EDITOR_DESTROYED) {
     NS_WARNING(
         "CSSEditUtils::RemoveCSSPropertyWithTransaction(nsGkAtoms::z_index) "
@@ -772,8 +795,8 @@ nsresult HTMLEditor::SetPositionToStatic(Element& aElement) {
   if (!HTMLEditUtils::IsImage(styledElement)) {
     // MOZ_KnownLive(*styledElement): aElement's lifetime must be guarantted
     // by the caller because of MOZ_CAN_RUN_SCRIPT method.
-    rv = mCSSEditUtils->RemoveCSSPropertyWithTransaction(
-        MOZ_KnownLive(*styledElement), *nsGkAtoms::width, u""_ns);
+    rv = CSSEditUtils::RemoveCSSPropertyWithTransaction(
+        *this, MOZ_KnownLive(*styledElement), *nsGkAtoms::width, u""_ns);
     if (rv == NS_ERROR_EDITOR_DESTROYED) {
       NS_WARNING(
           "CSSEditUtils::RemoveCSSPropertyWithTransaction(nsGkAtoms::width) "
@@ -786,8 +809,8 @@ nsresult HTMLEditor::SetPositionToStatic(Element& aElement) {
         "failed, but ignored");
     // MOZ_KnownLive(*styledElement): aElement's lifetime must be guarantted
     // by the caller because of MOZ_CAN_RUN_SCRIPT method.
-    rv = mCSSEditUtils->RemoveCSSPropertyWithTransaction(
-        MOZ_KnownLive(*styledElement), *nsGkAtoms::height, u""_ns);
+    rv = CSSEditUtils::RemoveCSSPropertyWithTransaction(
+        *this, MOZ_KnownLive(*styledElement), *nsGkAtoms::height, u""_ns);
     if (rv == NS_ERROR_EDITOR_DESTROYED) {
       NS_WARNING(
           "CSSEditUtils::RemoveCSSPropertyWithTransaction(nsGkAtoms::height) "
@@ -805,30 +828,60 @@ nsresult HTMLEditor::SetPositionToStatic(Element& aElement) {
     return NS_OK;
   }
 
+  EditorDOMPoint pointToPutCaret;
   // Make sure the first fild and last child of aElement starts/ends hard
   // line(s) even after removing `aElement`.
-  // MOZ_KnownLive(*styledElement): aElement's lifetime must be guarantted
-  // by the caller because of MOZ_CAN_RUN_SCRIPT method.
-  rv = EnsureHardLineBeginsWithFirstChildOf(MOZ_KnownLive(*styledElement));
-  if (NS_FAILED(rv)) {
-    NS_WARNING("HTMLEditor::EnsureHardLineBeginsWithFirstChildOf() failed");
-    return rv;
+  {
+    // MOZ_KnownLive(*styledElement): aElement's lifetime must be guarantted
+    // by the caller because of MOZ_CAN_RUN_SCRIPT method.
+    Result<CreateElementResult, nsresult>
+        maybeInsertBRElementBeforeFirstChildResult =
+            EnsureHardLineBeginsWithFirstChildOf(MOZ_KnownLive(*styledElement));
+    if (MOZ_UNLIKELY(maybeInsertBRElementBeforeFirstChildResult.isErr())) {
+      NS_WARNING("HTMLEditor::EnsureHardLineBeginsWithFirstChildOf() failed");
+      return maybeInsertBRElementBeforeFirstChildResult.unwrapErr();
+    }
+    CreateElementResult unwrappedResult =
+        maybeInsertBRElementBeforeFirstChildResult.unwrap();
+    if (unwrappedResult.HasCaretPointSuggestion()) {
+      pointToPutCaret = unwrappedResult.UnwrapCaretPoint();
+    }
   }
-  // MOZ_KnownLive(*styledElement): aElement's lifetime must be guarantted
-  // by the caller because of MOZ_CAN_RUN_SCRIPT method.
-  rv = EnsureHardLineEndsWithLastChildOf(MOZ_KnownLive(*styledElement));
-  if (NS_FAILED(rv)) {
-    NS_WARNING("HTMLEditor::EnsureHardLineEndsWithLastChildOf() failed");
-    return rv;
+  {
+    // MOZ_KnownLive(*styledElement): aElement's lifetime must be guarantted
+    // by the caller because of MOZ_CAN_RUN_SCRIPT method.
+    Result<CreateElementResult, nsresult>
+        maybeInsertBRElementAfterLastChildResult =
+            EnsureHardLineEndsWithLastChildOf(MOZ_KnownLive(*styledElement));
+    if (MOZ_UNLIKELY(maybeInsertBRElementAfterLastChildResult.isErr())) {
+      NS_WARNING("HTMLEditor::EnsureHardLineEndsWithLastChildOf() failed");
+      return maybeInsertBRElementAfterLastChildResult.unwrapErr();
+    }
+    CreateElementResult unwrappedResult =
+        maybeInsertBRElementAfterLastChildResult.unwrap();
+    if (unwrappedResult.HasCaretPointSuggestion()) {
+      pointToPutCaret = unwrappedResult.UnwrapCaretPoint();
+    }
   }
-  // MOZ_KnownLive(*styledElement): aElement's lifetime must be guarantted
-  // by the caller because of MOZ_CAN_RUN_SCRIPT method.
-  rv = RemoveContainerWithTransaction(MOZ_KnownLive(*styledElement));
-  if (NS_WARN_IF(Destroyed())) {
-    return NS_ERROR_EDITOR_DESTROYED;
+  {
+    // MOZ_KnownLive(*styledElement): aElement's lifetime must be guarantted
+    // by the caller because of MOZ_CAN_RUN_SCRIPT method.
+    Result<EditorDOMPoint, nsresult> unwrapStyledElementResult =
+        RemoveContainerWithTransaction(MOZ_KnownLive(*styledElement));
+    if (MOZ_UNLIKELY(unwrapStyledElementResult.isErr())) {
+      NS_WARNING("HTMLEditor::RemoveContainerWithTransaction() failed");
+      return unwrapStyledElementResult.unwrapErr();
+    }
+    if (unwrapStyledElementResult.inspect().IsSet()) {
+      pointToPutCaret = unwrapStyledElementResult.unwrap();
+    }
   }
+  if (!AllowsTransactionsToChangeSelection() || !pointToPutCaret.IsSet()) {
+    return NS_OK;
+  }
+  rv = CollapseSelectionTo(pointToPutCaret);
   NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
-                       "HTMLEditor::RemoveContainerWithTransaction() failed");
+                       "EditorBase::CollapseSelectionTo() failed");
   return rv;
 }
 
@@ -854,11 +907,11 @@ NS_IMETHODIMP HTMLEditor::GetGridSize(uint32_t* aSize) {
 
 nsresult HTMLEditor::SetTopAndLeftWithTransaction(
     nsStyledElement& aStyledElement, int32_t aX, int32_t aY) {
-  AutoPlaceholderBatch treatAsOneTransaction(*this,
-                                             ScrollSelectionIntoView::Yes);
+  AutoPlaceholderBatch treatAsOneTransaction(
+      *this, ScrollSelectionIntoView::Yes, __FUNCTION__);
   nsresult rv;
-  rv = mCSSEditUtils->SetCSSPropertyPixelsWithTransaction(aStyledElement,
-                                                          *nsGkAtoms::left, aX);
+  rv = CSSEditUtils::SetCSSPropertyPixelsWithTransaction(*this, aStyledElement,
+                                                         *nsGkAtoms::left, aX);
   if (rv == NS_ERROR_EDITOR_DESTROYED) {
     NS_WARNING(
         "CSSEditUtils::SetCSSPropertyPixelsWithTransaction(nsGkAtoms::left) "
@@ -869,8 +922,8 @@ nsresult HTMLEditor::SetTopAndLeftWithTransaction(
       NS_SUCCEEDED(rv),
       "CSSEditUtils::SetCSSPropertyPixelsWithTransaction(nsGkAtoms::left) "
       "failed, but ignored");
-  rv = mCSSEditUtils->SetCSSPropertyPixelsWithTransaction(aStyledElement,
-                                                          *nsGkAtoms::top, aY);
+  rv = CSSEditUtils::SetCSSPropertyPixelsWithTransaction(*this, aStyledElement,
+                                                         *nsGkAtoms::top, aY);
   if (rv == NS_ERROR_EDITOR_DESTROYED) {
     NS_WARNING(
         "CSSEditUtils::SetCSSPropertyPixelsWithTransaction(nsGkAtoms::top) "
@@ -924,8 +977,8 @@ nsresult HTMLEditor::GetTemporaryStyleForFocusedPositionedElement(
     return NS_OK;
   }
 
-  RefPtr<ComputedStyle> style =
-      nsComputedDOMStyle::GetComputedStyle(&aElement, nullptr);
+  RefPtr<const ComputedStyle> style =
+      nsComputedDOMStyle::GetComputedStyle(&aElement);
   if (NS_WARN_IF(Destroyed())) {
     return NS_ERROR_EDITOR_DESTROYED;
   }
@@ -936,9 +989,10 @@ nsresult HTMLEditor::GetTemporaryStyleForFocusedPositionedElement(
 
   static const uint8_t kBlackBgTrigger = 0xd0;
 
-  const auto& color = style->StyleText()->mColor;
-  if (color.red >= kBlackBgTrigger && color.green >= kBlackBgTrigger &&
-      color.blue >= kBlackBgTrigger) {
+  auto color = style->StyleText()->mColor.ToColor();
+  if (NS_GET_R(color) >= kBlackBgTrigger &&
+      NS_GET_G(color) >= kBlackBgTrigger &&
+      NS_GET_B(color) >= kBlackBgTrigger) {
     aReturn.AssignLiteral("black");
   } else {
     aReturn.AssignLiteral("white");
