@@ -5,8 +5,16 @@
 
 #include "SelectionState.h"
 
-#include "mozilla/Assertions.h"   // for MOZ_ASSERT, etc.
-#include "mozilla/EditorUtils.h"  // for EditorUtils
+#include "AutoClonedRangeArray.h"  // for AutoClonedRangeArray
+#include "EditorUtils.h"           // for EditorUtils
+#include "EditorLineBreak.h"       // for EditorLineBreak
+#include "HTMLEditHelpers.h"       // for DeleteRangeResult
+
+#include "ErrorList.h"
+#include "mozilla/Assertions.h"    // for MOZ_ASSERT, etc.
+#include "mozilla/IntegerRange.h"  // for IntegerRange
+#include "mozilla/Likely.h"        // For MOZ_LIKELY and MOZ_UNLIKELY
+#include "mozilla/RangeUtils.h"    // for RangeUtils
 #include "mozilla/dom/RangeBinding.h"
 #include "mozilla/dom/Selection.h"  // for Selection
 #include "nsAString.h"              // for nsAString::Length
@@ -21,6 +29,23 @@ namespace mozilla {
 
 using namespace dom;
 
+/*****************************************************************************
+ * mozilla::RangeItem
+ *****************************************************************************/
+
+nsINode* RangeItem::GetRoot() const {
+  if (MOZ_UNLIKELY(!IsPositioned())) {
+    return nullptr;
+  }
+  nsINode* rootNode = RangeUtils::ComputeRootNode(mStartContainer);
+  if (mStartContainer == mEndContainer) {
+    return rootNode;
+  }
+  return MOZ_LIKELY(rootNode == RangeUtils::ComputeRootNode(mEndContainer))
+             ? rootNode
+             : nullptr;
+}
+
 /******************************************************************************
  * mozilla::SelectionState
  *
@@ -36,7 +61,15 @@ template nsresult RangeUpdater::SelAdjInsertNode(const EditorDOMPoint& aPoint);
 template nsresult RangeUpdater::SelAdjInsertNode(
     const EditorRawDOMPoint& aPoint);
 
-SelectionState::SelectionState() : mDirection(eDirNext) {}
+SelectionState::SelectionState(const AutoClonedSelectionRangeArray& aRanges)
+    : mDirection(aRanges.GetDirection()) {
+  mArray.SetCapacity(aRanges.Ranges().Length());
+  for (const OwningNonNull<nsRange>& range : aRanges.Ranges()) {
+    RefPtr<RangeItem> rangeItem = new RangeItem();
+    rangeItem->StoreRange(range);
+    mArray.AppendElement(std::move(rangeItem));
+  }
+}
 
 void SelectionState::SaveSelection(Selection& aSelection) {
   // if we need more items in the array, new them
@@ -51,9 +84,12 @@ void SelectionState::SaveSelection(Selection& aSelection) {
   }
 
   // now store the selection ranges
-  for (uint32_t i = 0; i < aSelection.RangeCount(); i++) {
+  const uint32_t rangeCount = aSelection.RangeCount();
+  for (const uint32_t i : IntegerRange(rangeCount)) {
+    MOZ_ASSERT(aSelection.RangeCount() == rangeCount);
     const nsRange* range = aSelection.GetRangeAt(i);
-    if (NS_WARN_IF(!range)) {
+    MOZ_ASSERT(range);
+    if (MOZ_UNLIKELY(NS_WARN_IF(!range))) {
       continue;
     }
     mArray[i]->StoreRange(*range);
@@ -89,19 +125,19 @@ nsresult SelectionState::RestoreSelection(Selection& aSelection) {
   return NS_OK;
 }
 
-bool SelectionState::IsCollapsed() const {
-  if (mArray.Length() != 1) {
-    return false;
+void SelectionState::ApplyTo(AutoClonedSelectionRangeArray& aRanges) {
+  aRanges.RemoveAllRanges();
+  aRanges.SetDirection(mDirection);
+  for (const RefPtr<RangeItem>& rangeItem : mArray) {
+    RefPtr<nsRange> range = rangeItem->GetRange();
+    if (MOZ_UNLIKELY(!range)) {
+      continue;
+    }
+    aRanges.Ranges().AppendElement(std::move(range));
   }
-  RefPtr<nsRange> range = mArray[0]->GetRange();
-  if (!range) {
-    NS_WARNING("RangeItem::GetRange() failed");
-    return false;
-  }
-  return range->Collapsed();
 }
 
-bool SelectionState::Equals(SelectionState& aOther) const {
+bool SelectionState::Equals(const SelectionState& aOther) const {
   if (mArray.Length() != aOther.mArray.Length()) {
     return false;
   }
@@ -112,40 +148,9 @@ bool SelectionState::Equals(SelectionState& aOther) const {
     return false;
   }
 
-  // XXX Creating nsRanges are really expensive.  Why cannot we just check
-  //     the container and offsets??
-  IgnoredErrorResult ignoredError;
-  for (size_t i = 0; i < mArray.Length(); i++) {
-    RefPtr<nsRange> range = mArray[i]->GetRange();
-    if (!range) {
-      NS_WARNING("Failed to create a range from the range item");
-      return false;
-    }
-    RefPtr<nsRange> otherRange = aOther.mArray[i]->GetRange();
-    if (!otherRange) {
-      NS_WARNING("Failed to create a range from the other's range item");
-      return false;
-    }
-
-    int16_t compResult = range->CompareBoundaryPoints(
-        Range_Binding::START_TO_START, *otherRange, ignoredError);
-    if (ignoredError.Failed()) {
-      NS_WARNING(
-          "nsRange::CompareBoundaryPoints(Range_Binding::START_TO_START) "
-          "failed");
-      return false;
-    }
-    if (compResult) {
-      return false;
-    }
-    compResult = range->CompareBoundaryPoints(Range_Binding::END_TO_END,
-                                              *otherRange, ignoredError);
-    if (ignoredError.Failed()) {
-      NS_WARNING(
-          "nsRange::CompareBoundaryPoints(Range_Binding::END_TO_END) failed");
-      return false;
-    }
-    if (compResult) {
+  for (uint32_t i : IntegerRange(mArray.Length())) {
+    if (NS_WARN_IF(!mArray[i]) || NS_WARN_IF(!aOther.mArray[i]) ||
+        !mArray[i]->Equals(*aOther.mArray[i])) {
       return false;
     }
   }
@@ -153,14 +158,6 @@ bool SelectionState::Equals(SelectionState& aOther) const {
   return true;
 }
 
-void SelectionState::Clear() {
-  // free any items in the array
-  mArray.Clear();
-  mDirection = eDirNext;
-}
-
-bool SelectionState::IsEmpty() const { return mArray.IsEmpty(); }
-
 /******************************************************************************
  * mozilla::RangeUpdater
  *
@@ -302,8 +299,9 @@ void RangeUpdater::SelAdjDeleteNode(nsINode& aNodeToDelete) {
   }
 }
 
-nsresult RangeUpdater::SelAdjSplitNode(nsIContent& aRightNode,
-                                       nsIContent& aNewLeftNode) {
+nsresult RangeUpdater::SelAdjSplitNode(nsIContent& aOriginalContent,
+                                       uint32_t aSplitOffset,
+                                       nsIContent& aNewContent) {
   if (mLocked) {
     // lock set by Will/DidReplaceParent, etc...
     return NS_OK;
@@ -313,43 +311,56 @@ nsresult RangeUpdater::SelAdjSplitNode(nsIContent& aRightNode,
     return NS_OK;
   }
 
-  EditorRawDOMPoint atLeftNode(&aNewLeftNode);
-  nsresult rv = SelAdjInsertNode(atLeftNode);
-  if (NS_FAILED(rv)) {
-    NS_WARNING("RangeUpdater::SelAdjInsertNode() failed");
-    return rv;
+  EditorRawDOMPoint atNewNode(&aNewContent);
+  if (NS_WARN_IF(!atNewNode.IsSetAndValid())) {
+    return NS_ERROR_EDITOR_UNEXPECTED_DOM_TREE;
   }
 
-  // If point in the ranges is in left node, change its container to the left
-  // node.  If point in the ranges is in right node, subtract numbers of
-  // children moved to left node from the offset.
-  uint32_t lengthOfLeftNode = aNewLeftNode.Length();
+  auto AdjustDOMPoint = [&](nsCOMPtr<nsINode>& aContainer,
+                            uint32_t& aOffset) -> void {
+    if (aContainer == atNewNode.GetContainer()) {
+      // When we create a right node, we insert it after the left node.
+      // In this case,
+      // - `{}<left/>` should become `{}<left/><right/>` (0 -> 0)
+      // - `<left/>{}` should become `<left/><right/>{}` (1 -> 2)
+      // - `{<left/>}` should become `{<left/><right/>}` (0 -> 0, 1 -> 2}
+      // Therefore, we need to increate the offset only when the offset equals
+      // or is larger than the offset at the right node.
+      if (aOffset >= atNewNode.Offset()) {
+        aOffset++;
+      }
+    }
+    // If point is in the range which are moved from aOriginalContent to
+    // aNewContent, we need to change its container to aNewContent and may need
+    // to adjust the offset. If point is in the range which are not moved from
+    // aOriginalContent, we may need to adjust the offset.
+    if (aContainer != &aOriginalContent) {
+      return;
+    }
+    if (aOffset >= aSplitOffset) {
+      aContainer = &aNewContent;
+      aOffset -= aSplitOffset;
+    }
+  };
+
   for (RefPtr<RangeItem>& rangeItem : mArray) {
     if (NS_WARN_IF(!rangeItem)) {
       return NS_ERROR_FAILURE;
     }
-
-    if (rangeItem->mStartContainer == &aRightNode) {
-      if (rangeItem->mStartOffset > lengthOfLeftNode) {
-        rangeItem->mStartOffset -= lengthOfLeftNode;
-      } else {
-        rangeItem->mStartContainer = &aNewLeftNode;
-      }
-    }
-    if (rangeItem->mEndContainer == &aRightNode) {
-      if (rangeItem->mEndOffset > lengthOfLeftNode) {
-        rangeItem->mEndOffset -= lengthOfLeftNode;
-      } else {
-        rangeItem->mEndContainer = &aNewLeftNode;
-      }
-    }
+    AdjustDOMPoint(rangeItem->mStartContainer, rangeItem->mStartOffset);
+    AdjustDOMPoint(rangeItem->mEndContainer, rangeItem->mEndOffset);
   }
   return NS_OK;
 }
 
-nsresult RangeUpdater::SelAdjJoinNodes(nsINode& aLeftNode, nsINode& aRightNode,
-                                       nsINode& aParent, uint32_t aOffset,
-                                       uint32_t aOldLeftNodeLength) {
+nsresult RangeUpdater::SelAdjJoinNodes(
+    const EditorRawDOMPoint& aStartOfRightContent,
+    const nsIContent& aRemovedContent,
+    const EditorDOMPoint& aOldPointAtRightContent) {
+  MOZ_ASSERT(aStartOfRightContent.IsSetAndValid());
+  MOZ_ASSERT(aOldPointAtRightContent.IsSet());  // Invalid point in most cases
+  MOZ_ASSERT(aOldPointAtRightContent.HasOffset());
+
   if (mLocked) {
     // lock set by Will/DidReplaceParent, etc...
     return NS_OK;
@@ -359,44 +370,46 @@ nsresult RangeUpdater::SelAdjJoinNodes(nsINode& aLeftNode, nsINode& aRightNode,
     return NS_OK;
   }
 
+  auto AdjustDOMPoint = [&](nsCOMPtr<nsINode>& aContainer,
+                            uint32_t& aOffset) -> void {
+    // FYI: Typically, containers of aOldPointAtRightContent and
+    //      aStartOfRightContent are same.  They are different when one of the
+    //      node was moved to somewhere and they are joined by undoing splitting
+    //      a node.
+    if (aContainer == &aRemovedContent) {
+      // If the point is in the removed content, move the point to the new
+      // point in the joined node.  If left node content is moved into
+      // right node, the offset should be same.  Otherwise, we need to advance
+      // the offset to length of the removed content.
+      aContainer = aStartOfRightContent.GetContainer();
+      aOffset += aStartOfRightContent.Offset();
+    }
+    // TODO: If aOldPointAtRightContent.GetContainer() was in aRemovedContent,
+    //       we fail to adjust container and offset here because we need to
+    //       make point to where aRemoveContent was.  However, collecting all
+    //       ancestors of the right content may be expensive.  What's the best
+    //       approach to fix this?
+    else if (aContainer == aOldPointAtRightContent.GetContainer()) {
+      // If the point is in common parent of joined content nodes and it
+      // pointed after the right content node, decrease the offset.
+      if (aOffset > aOldPointAtRightContent.Offset()) {
+        aOffset--;
+      }
+      // If it pointed the right content node, adjust it to point ex-first
+      // content of the right node.
+      else if (aOffset == aOldPointAtRightContent.Offset()) {
+        aContainer = aStartOfRightContent.GetContainer();
+        aOffset = aStartOfRightContent.Offset();
+      }
+    }
+  };
+
   for (RefPtr<RangeItem>& rangeItem : mArray) {
     if (NS_WARN_IF(!rangeItem)) {
       return NS_ERROR_FAILURE;
     }
-
-    if (rangeItem->mStartContainer == &aParent) {
-      // adjust start point in aParent
-      if (rangeItem->mStartOffset > aOffset) {
-        rangeItem->mStartOffset--;
-      } else if (rangeItem->mStartOffset == aOffset) {
-        // join keeps right hand node
-        rangeItem->mStartContainer = &aRightNode;
-        rangeItem->mStartOffset = aOldLeftNodeLength;
-      }
-    } else if (rangeItem->mStartContainer == &aRightNode) {
-      // adjust start point in aRightNode
-      rangeItem->mStartOffset += aOldLeftNodeLength;
-    } else if (rangeItem->mStartContainer == &aLeftNode) {
-      // adjust start point in aLeftNode
-      rangeItem->mStartContainer = &aRightNode;
-    }
-
-    if (rangeItem->mEndContainer == &aParent) {
-      // adjust end point in aParent
-      if (rangeItem->mEndOffset > aOffset) {
-        rangeItem->mEndOffset--;
-      } else if (rangeItem->mEndOffset == aOffset) {
-        // join keeps right hand node
-        rangeItem->mEndContainer = &aRightNode;
-        rangeItem->mEndOffset = aOldLeftNodeLength;
-      }
-    } else if (rangeItem->mEndContainer == &aRightNode) {
-      // adjust end point in aRightNode
-      rangeItem->mEndOffset += aOldLeftNodeLength;
-    } else if (rangeItem->mEndContainer == &aLeftNode) {
-      // adjust end point in aLeftNode
-      rangeItem->mEndContainer = &aRightNode;
-    }
+    AdjustDOMPoint(rangeItem->mStartContainer, rangeItem->mStartOffset);
+    AdjustDOMPoint(rangeItem->mEndContainer, rangeItem->mEndOffset);
   }
 
   return NS_OK;
@@ -524,35 +537,34 @@ void RangeUpdater::DidRemoveContainer(const Element& aRemovedElement,
 
 void RangeUpdater::DidMoveNode(const nsINode& aOldParent, uint32_t aOldOffset,
                                const nsINode& aNewParent, uint32_t aNewOffset) {
-  if (NS_WARN_IF(!mLocked)) {
+  if (mLocked) {
+    // Do nothing if moving nodes is occurred while changing the container.
     return;
   }
-  mLocked = false;
-
+  auto AdjustDOMPoint = [&](nsCOMPtr<nsINode>& aNode, uint32_t& aOffset) {
+    if (aNode == &aOldParent) {
+      // If previously pointed the moved content, it should keep pointing it.
+      if (aOffset == aOldOffset) {
+        aNode = const_cast<nsINode*>(&aNewParent);
+        aOffset = aNewOffset;
+      } else if (aOffset > aOldOffset) {
+        aOffset--;
+      }
+      return;
+    }
+    if (aNode == &aNewParent) {
+      if (aOffset > aNewOffset) {
+        aOffset++;
+      }
+    }
+  };
   for (RefPtr<RangeItem>& rangeItem : mArray) {
     if (NS_WARN_IF(!rangeItem)) {
       return;
     }
 
-    // like a delete in aOldParent
-    if (rangeItem->mStartContainer == &aOldParent &&
-        rangeItem->mStartOffset > aOldOffset) {
-      rangeItem->mStartOffset--;
-    }
-    if (rangeItem->mEndContainer == &aOldParent &&
-        rangeItem->mEndOffset > aOldOffset) {
-      rangeItem->mEndOffset--;
-    }
-
-    // and like an insert in aNewParent
-    if (rangeItem->mStartContainer == &aNewParent &&
-        rangeItem->mStartOffset > aNewOffset) {
-      rangeItem->mStartOffset++;
-    }
-    if (rangeItem->mEndContainer == &aNewParent &&
-        rangeItem->mEndOffset > aNewOffset) {
-      rangeItem->mEndOffset++;
-    }
+    AdjustDOMPoint(rangeItem->mStartContainer, rangeItem->mStartOffset);
+    AdjustDOMPoint(rangeItem->mEndContainer, rangeItem->mEndOffset);
   }
 }
 
@@ -563,8 +575,6 @@ void RangeUpdater::DidMoveNode(const nsINode& aOldParent, uint32_t aOldOffset,
  ******************************************************************************/
 
 NS_IMPL_CYCLE_COLLECTION(RangeItem, mStartContainer, mEndContainer)
-NS_IMPL_CYCLE_COLLECTION_ROOT_NATIVE(RangeItem, AddRef)
-NS_IMPL_CYCLE_COLLECTION_UNROOT_NATIVE(RangeItem, Release)
 
 void RangeItem::StoreRange(const nsRange& aRange) {
   mStartContainer = aRange.GetStartContainer();
@@ -573,11 +583,65 @@ void RangeItem::StoreRange(const nsRange& aRange) {
   mEndOffset = aRange.EndOffset();
 }
 
-already_AddRefed<nsRange> RangeItem::GetRange() {
+already_AddRefed<nsRange> RangeItem::GetRange() const {
   RefPtr<nsRange> range = nsRange::Create(
       mStartContainer, mStartOffset, mEndContainer, mEndOffset, IgnoreErrors());
   NS_WARNING_ASSERTION(range, "nsRange::Create() failed");
   return range.forget();
 }
 
+/******************************************************************************
+ * mozilla::AutoTrackDOMPoint
+ ******************************************************************************/
+
+AutoTrackDOMPoint::AutoTrackDOMPoint(RangeUpdater& aRangeUpdater,
+                                     CaretPoint* aCaretPoint)
+    : AutoTrackDOMPoint(aRangeUpdater, &aCaretPoint->mCaretPoint) {}
+
+/******************************************************************************
+ * mozilla::AutoTrackDOMMoveNodeResult
+ ******************************************************************************/
+
+AutoTrackDOMMoveNodeResult::AutoTrackDOMMoveNodeResult(
+    RangeUpdater& aRangeUpdater, MoveNodeResult* aMoveNodeResult)
+    : mTrackCaretPoint(aRangeUpdater,
+                       static_cast<CaretPoint*>(aMoveNodeResult)),
+      mTrackNextInsertionPoint(aRangeUpdater,
+                               &aMoveNodeResult->mNextInsertionPoint),
+      mTrackMovedContentRange(aRangeUpdater,
+                              &aMoveNodeResult->mMovedContentRange) {}
+
+/******************************************************************************
+ * mozilla::AutoTrackDeleteRangeResult
+ ******************************************************************************/
+
+AutoTrackDOMDeleteRangeResult::AutoTrackDOMDeleteRangeResult(
+    RangeUpdater& aRangeUpdater, DeleteRangeResult* aDeleteRangeResult)
+    : mTrackCaretPoint(aRangeUpdater,
+                       static_cast<CaretPoint*>(aDeleteRangeResult)),
+      mTrackDeleteRange(aRangeUpdater, &aDeleteRangeResult->mDeleteRange) {}
+
+/******************************************************************************
+ * mozilla::AutoTrackLineBreak
+ ******************************************************************************/
+
+AutoTrackLineBreak::AutoTrackLineBreak(RangeUpdater& aRangeUpdater,
+                                       EditorLineBreak* aLineBreak)
+    : mLineBreak(aLineBreak->IsPreformattedLineBreak() ? aLineBreak : nullptr),
+      mPoint(mLineBreak ? mLineBreak->To<EditorDOMPoint>() : EditorDOMPoint()),
+      mTracker(aRangeUpdater, &mPoint) {
+  MOZ_ASSERT(aLineBreak->IsPreformattedLineBreak());
+}
+
+void AutoTrackLineBreak::FlushAndStopTracking() {
+  if (!mLineBreak) {
+    return;
+  }
+  mTracker.FlushAndStopTracking();
+  if (mPoint.GetContainer() == mLineBreak->mContent) {
+    mLineBreak->mOffsetInText = Some(mPoint.Offset());
+  }
+  mLineBreak = nullptr;
+}
+
 }  // namespace mozilla
