@@ -6,17 +6,15 @@
 
 #include <soundtouch/SoundTouch.h>
 
-namespace soundtouch
+extern "C" namespace soundtouch
 {
 
-SOUNDTOUCH_API
 soundtouch::SoundTouch*
 createSoundTouchObj()
 {
   return new soundtouch::SoundTouch();
 }
 
-SOUNDTOUCH_API
 void
 destroySoundTouchObj(soundtouch::SoundTouch* aObj)
 {
